/*
   3D Invaders

   invader.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "invader.h"

#define EXPLODE_FRMS 30
 
enum { I_ACTIVE, I_HIT, I_FALLING, I_EXPLODE, I_GONE };

int Invader::refcnt=0;
int Invader::sfx_fall;
PR_nurb * Invader::s1, * Invader::nrbR, * Invader::nrbs;
Ppoint4 * Invader::pts = NULL;
int Invader::nx, Invader::ny;
uv_t * Invader::uvs = NULL;

Invader::Invader() {
	velocity = -1.0f;
	mode = I_ACTIVE;
	rotx = roty = rotz = 0.0f;
	explode = roti = 0;
	sfxinit();
	make_nurbs();
	refcnt++;
}

void Invader::Draw(point_t * pos) {
	switch (mode) {
	case I_ACTIVE:
		position.x = pos->x;
		position.y = pos->y;
		position.z = pos->z;
		draw_active();
		break;
	case I_HIT:
		mode = I_FALLING;
		snd_sfx_play(sfx_invader_hit, gbl_sfx_volume, 128);
		explode = EXPLODE_FRMS;
	case I_EXPLODE:
	case I_FALLING:
		mode = draw_falling();
		break;
	case I_GONE:
		break;
	default:
		assert_msg(0, "Unknown Invader mode");
	}
}

int Invader::Active() {
	return !(mode == I_GONE)? -1 : 0;
}

void Invader::Hit() {
	mode = I_HIT;
	hits--;
}

void Invader::Poof() {
	mode = I_GONE;
	position.z = 1050.0f;
}

void Invader::Reset() {
	mode = I_ACTIVE;
	rotx = roty = rotz = 0.0f;
	explode = 0;
	hits = MAX_HITS;
}

void Invader::draw_active() {
	// draw invader 
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, txinvaders); 
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotatef(rotx+=1.0f, 0.0f, 0.0f, -1.0f);
	draw_invader();
	glPopMatrix();
}

int Invader::draw_falling() {
	// draw falling invader 
	if (((explode%4) < 2)) {
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, txinvaders); 
		glPushMatrix();
		glTranslatef(position.x, position.y, position.z);
		glRotatef(rotx+=3.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(roty+=4.0f, 0.0f, 1.0f, 0.0f);
		draw_invader();
		glPopMatrix();
	}
	// calculate next position
	position.z += 0.2f; 
	if ((position.z < 20.0f) && (hits > 0)) {
		if (--explode > 0)
			return I_EXPLODE;
		else
			return I_FALLING;
	} else {
		position.z = 20.0f;
		return I_GONE;
	}
}

void Invader::make_nurbs() {
	int i, j, jj;
	Ppoint3 v1;
	Pmatrix3 xx;

	if (!refcnt) {
		/* create control points for the object */
		s1 = NULL;
		nrbR = nrbs = NULL;
		nrb_allocatenurb(&s1);
		nrb_allocatenurb(&nrbR);
		nrb_allocatenurb(&nrbs);
		
		/* use a torus as our starting shape */
		nrb_torus(s1) ;
  
		/* stretch it in the z-axis */
		v1 = ptk_point3(0.5,0.5,0.5);
		ptk_scale3(&v1,PREPLACE,xx);
		nrb_xform(s1,xx);
  
		/* force all x and y coordinates
		   to be the same point.
		*/           
		/* for( i = 0 ; i < s1->pf_u.pf_n ; i++ ) {
			s1->pf_ppp->pts[i].x =  0.0 ;
			s1->pf_ppp->pts[i].y =  0.0 ;
		} */
	
		/* refine to full size */
		nx = nrb_numsubdivs(s1, PVudir, 0.4);
		ny = nrb_numsubdivs(s1, PVvdir, 0.4);
		nrb_tessalate(s1, nx, nrbR);
		nrb_tessalate(nrbR, ny, nrbs);
		
		pts = nrbs->pf_ppp->pts;
		nx = nrbs->pf_u.pf_n;
		ny = nrbs->pf_v.pf_n;
		printf("Invader nx=%d, ny=%d, polys=%d\n",nx, ny, nx*ny*2);

		/* release all ram except the final surface itself */
		nrb_deallocatenurb(&s1);
		nrb_deallocatenurb(&nrbR);

		//Map a texture to the surface
		uvs = (uv_t*) malloc(nx*nx*sizeof(uv_t));
		for (j = 0; j < ny; j++) {
			for (i = 0; i < nx; i++) {
				jj = i + (j*nx);
				uvs[jj].u = ((float)i/(float)nx);
				uvs[jj].v = (((float)(j)/(float)(ny-1)));
			}
		}
	}
}

void Invader::draw_invader() {
	int i, j, jj, cull;
	float x, y, z, u, v, w;
	float lx=0.0f, ly=0.0f, lz=0.0f, lu=0.0f, lv=0.0f;
	pvr_vertex_t	*vert;
	pvr_dr_state_t	dr_state;
	
	pvr_dr_init(dr_state);
	
	/* Initialize xmtrx with the values from KGL */
	glKosMatrixIdent();
	glKosMatrixApply(GL_KOS_SCREENVIEW);
	glKosMatrixApply(GL_PROJECTION);
	glKosMatrixApply(GL_MODELVIEW);

	glKosPolyHdrSend();

	//Draw nurbs surface using a series of strips
	for (j = 0; j < ny-1; j++) {
		cull = false;
		for (i = 0; i <= nx; i++) {
			jj = (i-1*(i%2)) + (j*nx) + (nx * (i%2));
			w = 1.0f / pts[jj].w;
			vert = pvr_dr_target(dr_state);
			if (i == nx) {
				vert->flags = PVR_CMD_VERTEX_EOL;
			} else
				vert->flags = PVR_CMD_VERTEX;
			u = uvs[jj].u;
			v = uvs[jj].v;
			x = pts[jj].x*w;
			y = pts[jj].y*w;
			z = pts[jj].z*w;
			mat_trans_single(x, y, z);
			if ( z < 0.0f || cull) {
				cull = true;
				vert->x = lx;
				vert->y = ly;
				vert->z = lz;
				vert->u = lu;
				vert->v = lv;
				vert->argb = 0xffffffff;
			} else {
				vert->x = lx = x;
				vert->y = ly = y;
				vert->z = lz = z;
				vert->u = lu = u;
				vert->v = lv = v;
				vert->argb = 0xffffffff;
			}
			pvr_dr_commit(vert);
		}
	}
}
