/*
   3D Invaders

   main.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "stars.h"
#include "plprint.h"

#undef VIDBUG

Game * game;
Shots * shots[MAX_SHOTS];
Bombs * bombs[MAX_BOMBS];
Player * player;
Invader * invaders[IROWS*ICOLS];
Barrier * barriers[BARRIERS];

void back_one_frame() {
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_KOS_AUTO_UV);
	glBindTexture( GL_TEXTURE_2D, tx0121w);
	glPushMatrix();
        glTranslatef(0.0f, 0.0f, -200.0f);
        //glRotatef((float)rz/10, 0.0f, 0.0f, 0.01f);
        glScalef(200.0f, 200.0f, 20.0f);
	glBegin(GL_QUADS);
	glColor3f(0.5f, 0.5f, 0.5f);
	/* Front face */
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);
	glEnd();
	glPopMatrix();
	glDisable(GL_KOS_AUTO_UV);
}

void init_ordinance() {
	int i;
	//shots = (Shots**) malloc(MAX_SHOTS*sizeof(Shots*));
	for (i=0; i<MAX_SHOTS; i++) {
		shots[i] = (Shots *) new Shots();
	}

	for (i=0; i<MAX_BOMBS; i++) {
		bombs[i] = (Bombs *) new Bombs();
	}
}

void do_text() {
        plprint(1, 1, 0.3f, 0.7f, 0.9f,
                        " 3D Invaders pre-release Rev A by Axlen", 0);
        plprint(1, 3, 0.3f, 0.7f, 0.9f,
                        "    For the DCEmu Programming Contest", 0);
	plprint(1, 28, 0.3f, 0.7f, 0.9f,
                        "[A] Fire   [D-PAD] Move Player  [Y] Reset", 0);
	plprint(1, 30, 0.3f, 0.7f, 0.9f,
                        "(c)2002 Paul Boese      KOS-CVS 5 May 2002", 0);
}

float player_ry() {
	point_t *pp = player->Pos();
	return fsin(pp->y*F_PI/180.0f);
}

float player_rx() {
	point_t *pp = player->Pos();
	return fsin(pp->x*F_PI/180.0f);
}

extern "C" uint8 romdisk[];
int main(int argc, char **argv) {
	uint8 c;
	cont_cond_t cond;
	int i, j=0;
	GLboolean A = GL_FALSE;
	GLboolean Y = GL_FALSE;
	
	kos_init_all(BASIC_ENABLE | THD_ENABLE, romdisk);

	if (gfx_init() < 0)
		return -1;

	if (snd_init() < 0)
		return -1;

	// this be ugly
	plinit();
	stars_init();
	init_ordinance();
	game = (Game *) new Game();
	player = (Player *) new Player();
	for(int i=0; i<(IROWS*ICOLS); i++) {
		invaders[i] = (Invader *) new Invader();
	}
	for(int i=0; i<BARRIERS; i++) {
		barriers[i] = (Barrier *) new Barrier();
	}
	game->doInitLevel(invaders, barriers, player);

	printf("\n[... 3D Invaders START .................................]\n");

	/* do nothing.... yet */
	c = maple_first_controller();

	//snd_test("/rd/test.ogg");
	snd_volume(gbl_ogg_volume);

	while(1) {
		if (cont_get_cond(c, &cond) < 0)
			break;
		if (!(cond.buttons & CONT_START))
			break;
		
		if (cond.buttons & CONT_DPAD_UP)
			player->Up();

		if (cond.buttons & CONT_DPAD_DOWN)
			player->Down();

		if (cond.buttons & CONT_DPAD_LEFT)
			player->Left();

		if (cond.buttons & CONT_DPAD_RIGHT)
			player->Right();

		if (!(cond.buttons & CONT_A) && !A) {
			shots[j%MAX_SHOTS]->Fire(player->Pos());
			j++;
			A = GL_TRUE;
		}	
		if (cond.buttons & CONT_A)
			A = GL_FALSE;
		
		if (!(cond.buttons & CONT_Y) && !Y) {
			/* XXX TEST */
			game->doInitLevel(invaders, barriers, player);
			Y = GL_TRUE;
		}	
		if (cond.buttons & CONT_Y)
			Y = GL_FALSE;
		glKosBeginFrame();
#ifdef VIDBUG
vid_border_color(0,255,0);	
#endif
		glLoadIdentity();
		/* gluLookAt( 0.0f, 5.0f, 8.0f,
				0.0f, 2.0f, 0.0f,
				0.0f, 1.0f, 0.0f); */

		/* gluLookAt( 0.0f, 0.0f, 15.0f, // okay
				0.0f, 0.0f, 0.0f,
				0.0f, 1.0f, 0.0f); */

		gluLookAt( 15.0f*player_rx(), 15.0f*player_ry(), 15.0f,
				400.0f*player_rx(), 400.0f*player_ry(), -400.0f,
				0.0f, 1.0f, 0.0f);


		//stars_one_frame();
		back_one_frame();
		game->doInvaders(invaders, shots, bombs);
		//game->doPlayer(player, invaders, bombs);

		glKosFinishList();
		for(i=0; i<MAX_SHOTS; i++)
			shots[i]->Fired();
		for(i=0; i<MAX_BOMBS; i++)
			bombs[i]->Dropped();

		game->doBarrier(barriers, invaders, shots, bombs);
		game->doPlayer(player, invaders, bombs);
		do_text();
#ifdef VIDBUG
vid_border_color(255,0,0);
#endif
		glKosFinishFrame();
#ifdef VIDBUG
vid_border_color( 0,0,255);	
#endif
	}

	printf("\n[... 3D Invaders FINISH ................................]\n\n");
	
	stars_shutdown();
	snd_shutdown();
	gfx_shutdown();
	kos_shutdown_all();
	return 0;
}
