/*
   3D Invaders

   shots.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "shots.h"

#define RELOAD_TICKS 60 

enum { S_LOADED, S_FIRE, S_FLY, S_HIT, S_MISS, S_RELOAD };

int Shots::refcnt=0;

Shots::Shots() {
	velocity = -1.0f;
	smode = S_LOADED;
	rotf = 0.0f;
	roti = 0;
	refcnt++;
	reload_ticks=0;
}

void Shots::Fire(point_t * initial_pos) {
	if (smode == S_LOADED) {
		position.x = initial_pos->x;
		position.y = initial_pos->y;
		position.z = initial_pos->z;
		velocity = 0;
		smode = S_FIRE;
		reload_ticks = RELOAD_TICKS;
	} else {
		// click sound effect
		snd_sfx_play(sfx_shot_click, gbl_sfx_volume, 128); 
	}
}

void Shots::Fired() {
	if (reload_ticks)
		reload_ticks--;
	
	switch (smode) {
	case S_LOADED:
		break;
	case S_FIRE:
		smode = S_FLY;
		// fire sound effect
		snd_sfx_play(sfx_shot_fired, gbl_sfx_volume, 128); 
	case S_FLY:
		smode = draw_flight();
		break;
	case S_HIT:
	case S_MISS:
		smode = S_RELOAD;
	case S_RELOAD:
		if (!reload_ticks) {
			smode = S_LOADED;
		}
		break;
	default:
		assert_msg(0, "Unknown Shots mode");
	}
}

int Shots::Active() {
	return (smode == S_FLY)? -1 : 0;
}

void Shots::Hit() {
	smode = S_HIT;
}

int Shots::draw_flight() {
	// draw missle object
	static float frame[16][2] = {
		{0.75f, 0.0f}, {0.75f, 0.25f}, {0.75f, 0.5f}, {0.75f, 0.75f},
		{0.5f,  0.0f}, {0.5f,  0.25f}, {0.5f,  0.5f}, {0.5f,  0.75f},
		{0.25f, 0.0f}, {0.25f, 0.25f}, {0.25f, 0.5f}, {0.25f, 0.75f},
		{0.0f,  0.0f}, {0.0f,  0.25f}, {0.0f,  0.5f}, {0.0f,  0.75f}
	};
	GLint idx=(roti++)%16;
	GLfloat u=frame[idx][0], v=frame[idx][1];
	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glBindTexture(GL_TEXTURE_2D, txshots);
	glBegin(GL_QUADS);
	glColor3f(1.0, 1.0, 1.0);
	glTexCoord2f(u,       v      ); glVertex3f(-1.0, -1.0, 0.0);
	glTexCoord2f(u+0.25f, v      ); glVertex3f( 1.0, -1.0, 0.0);
	glTexCoord2f(u+0.25f, v+0.25f); glVertex3f( 1.0,  1.0, 0.0);
	glTexCoord2f(u,       v+0.25f); glVertex3f(-1.0,  1.0, 0.0);
	glEnd();
	glPopMatrix();
	// calculate next position
	position.z += -1.0f; 
	if (position.z > -90.0f) {
		return S_FLY;
	}
	else {
		return S_MISS;
	}
}
